'use strict';

var generateScriptableBanner = require('@scriptables/manifest/generateScriptableBanner');
var fs = require('fs');
var promises = require('fs/promises');
var micromatch = require('micromatch');
var path = require('path');

function _interopDefault (e) { return e && e.__esModule ? e : { default: e }; }

var micromatch__default = /*#__PURE__*/_interopDefault(micromatch);

// src/scriptable-banner.ts

// src/logger.ts
var levels = {
  verbose: 0,
  info: 1,
  warn: 2,
  error: 3
};
var Logger = class {
  namespace;
  level;
  enabled;
  constructor(namespace, options = {}) {
    this.namespace = namespace;
    this.level = options.level || "info";
    this.enabled = options.enabled ?? true;
  }
  shouldLog(msgLevel) {
    return this.enabled && levels[msgLevel] >= levels[this.level];
  }
  verbose(...args) {
    if (this.shouldLog("verbose")) {
      console.log(`[${this.namespace}]`, ...args);
    }
  }
  info(...args) {
    if (this.shouldLog("info")) {
      console.info(`[${this.namespace}]`, ...args);
    }
  }
  warn(...args) {
    if (this.shouldLog("warn")) {
      console.warn(`[${this.namespace}]`, ...args);
    }
  }
  error(...args) {
    if (this.shouldLog("error")) {
      console.error(`[${this.namespace}]`, ...args);
    }
  }
};
function createLogger(namespace, options) {
  return new Logger(namespace, options);
}

// src/scriptable-banner.ts
var DEFAULT_MANIFEST_EXTENSIONS = [".manifest.json", ".manifest", ".json"];
function scriptableBanner(options = {}) {
  const {
    manifestExtensions = DEFAULT_MANIFEST_EXTENSIONS,
    warnOnMissingManifest = true,
    warnOnInvalidManifest = true,
    warnOnMissingEntry = true,
    getManifestPath,
    patterns = ["**/*.{ts,tsx,js,jsx}"]
  } = options;
  const logger = createLogger("scriptable-banner");
  return {
    name: "scriptable",
    setup(build) {
      let manifestCache = /* @__PURE__ */ new Map();
      if (!build.initialOptions.metafile) {
        logger.warn(
          'Warning: "metafile" option was not enabled. Automatically setting "metafile: true" to allow scriptable plugin to modify output files.'
        );
        build.initialOptions.metafile = true;
      }
      build.onStart(() => {
        manifestCache.clear();
      });
      build.onResolve({ filter: /.*/ }, async (args) => {
        if (!micromatch__default.default.isMatch(args.path, patterns)) {
          return null;
        }
        if (Array.isArray(build.initialOptions.entryPoints) && build.initialOptions.entryPoints.map(String).includes(args.path)) {
          const dir = path.dirname(args.path);
          const baseFileName = path.basename(args.path).replace(/\.[tj]sx?$/, "");
          const manifestPath = getManifestPath ? getManifestPath(args.path) : manifestExtensions.map((ext) => path.resolve(dir, `${baseFileName}${ext}`)).find(fs.existsSync);
          if (manifestPath) {
            try {
              const manifestContent = await promises.readFile(manifestPath, "utf8");
              const manifest = JSON.parse(manifestContent);
              const normalizedPath = path.posix.normalize(args.path.split(path.sep).join("/"));
              if (!manifest.name || !manifest.author || !manifest.version) {
                if (warnOnInvalidManifest) {
                  logger.warn(`Invalid manifest file: ${manifestPath} - missing required fields`);
                }
                return null;
              }
              manifestCache.set(normalizedPath, manifest);
            } catch (error) {
              logger.warn(`Failed to read manifest file: ${manifestPath}`, error);
            }
          } else if (warnOnMissingManifest) {
            logger.warn(`No manifest file found for entry point: ${args.path}`);
          }
        }
        return null;
      });
      build.onEnd(async (result) => {
        if (!result.metafile || Object.keys(result.metafile.outputs).length === 0) {
          logger.warn(
            'Skipping banner addition because "metafile" is not available or contains no outputs. Ensure "metafile: true" is set in esbuild options if write is enabled.'
          );
          return;
        }
        const outputs = result.metafile.outputs;
        for (const outputPath of Object.keys(outputs)) {
          if (build.initialOptions.write !== false && !fs.existsSync(outputPath)) {
            logger.warn(`Output file does not exist: ${outputPath}`);
            continue;
          }
          const outputMeta = outputs[outputPath];
          const entryPoint = outputMeta?.entryPoint ?? "";
          const normalizedEntryPath = entryPoint ? path.posix.normalize(entryPoint.split(path.sep).join("/")) : "";
          if (!normalizedEntryPath || !manifestCache.has(normalizedEntryPath)) {
            if (warnOnMissingEntry) {
              logger.warn(`No matching entry point found for output file: ${outputPath}`);
            }
            continue;
          }
          try {
            const manifest = manifestCache.get(normalizedEntryPath) || {};
            const banner = generateScriptableBanner.generateScriptableBanner(manifest);
            if (result.outputFiles) {
              const outputFile = result.outputFiles.find((file) => file.path === outputPath || file.path === "<stdout>");
              if (outputFile) {
                outputFile.contents = Buffer.from(banner + outputFile.text.trimEnd());
              }
            } else {
              const content = await promises.readFile(outputPath, "utf-8");
              const updatedContent = banner + content;
              await promises.writeFile(outputPath, updatedContent, "utf8");
            }
          } catch (error) {
            logger.error(`Failed to process output file: ${outputPath}`, error);
          }
        }
      });
    }
  };
}

exports.scriptableBanner = scriptableBanner;
